@@ -231,7 +231,6 @@ cc_network_panel_dispose (GObject *object)
                 g_cancellable_cancel (priv->cancellable);
 
         g_clear_object (&priv->cancellable);
-        g_clear_object (&priv->rfkill_proxy);
         g_clear_object (&priv->builder);
         g_clear_object (&priv->client);
         g_clear_object (&priv->modem_manager);
@@ -472,10 +471,10 @@ object_removed_cb (NetObject *object, CcNetworkPanel *panel)
                 if (g_strcmp0 (net_object_get_id (object),
                                net_object_get_id (object_tmp)) == 0) {
                         g_object_unref (object_tmp);
-                        if (!gtk_list_store_remove (GTK_LIST_STORE (model), &iter))
-                                gtk_tree_model_get_iter_first (model, &iter);
-                        gtk_tree_selection_select_iter (selection, &iter);
-
+                        if (gtk_list_store_remove (GTK_LIST_STORE (model), &iter)) {
+                                if (gtk_tree_model_get_iter_first (model, &iter))
+                                        gtk_tree_selection_select_iter (selection, &iter);
+                        }
                         break;
                 }
                 g_object_unref (object_tmp);
@@ -1374,6 +1373,7 @@ cc_network_panel_init (CcNetworkPanel *panel)
         GtkWidget *widget;
         GtkWidget *toplevel;
         GDBusConnection *system_bus;
+        GtkCssProvider *provider;
 
         panel->priv = NETWORK_PANEL_PRIVATE (panel);
         g_resources_register (cc_network_get_resource ());
@@ -1477,4 +1477,12 @@ cc_network_panel_init (CcNetworkPanel *panel)
         widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder,
                                                      "vbox1"));
         gtk_container_add (GTK_CONTAINER (panel), widget);
+
+        provider = gtk_css_provider_new ();
+        gtk_css_provider_load_from_data (provider, ".circular-button { border-radius: 20px; outline-radius: 20px; }", -1, NULL);
+        gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
+                                                   GTK_STYLE_PROVIDER (provider),
+                                                   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+        g_object_unref (provider);
+
 }
