@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2010 Red Hat, Inc
  * Copyright (C) 2008 William Jon McCann <jmccann@redhat.com>
- * Copyright (C) 2010 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2010,2015 Richard Hughes <richard@hughsie.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -91,13 +91,17 @@ struct _CcPowerPanelPrivate
 
   GtkWidget     *automatic_suspend_row;
   GtkWidget     *automatic_suspend_label;
-  GtkWidget     *critical_battery_row;
 
   GDBusProxy    *bt_rfkill;
   GDBusProxy    *bt_properties;
   GtkWidget     *bt_switch;
   GtkWidget     *bt_row;
 
+  GDBusProxy    *iio_proxy;
+  guint          iio_proxy_watch_id;
+  GtkWidget     *als_switch;
+  GtkWidget     *als_row;
+
 #ifdef HAVE_NETWORK_MANAGER
   NMClient      *nm_client;
   GtkWidget     *wifi_switch;
@@ -140,11 +144,15 @@ cc_power_panel_dispose (GObject *object)
   g_clear_object (&priv->up_client);
   g_clear_object (&priv->bt_rfkill);
   g_clear_object (&priv->bt_properties);
+  g_clear_object (&priv->iio_proxy);
 #ifdef HAVE_NETWORK_MANAGER
   g_clear_object (&priv->nm_client);
 #endif
   g_clear_pointer (&priv->boxes, g_list_free);
   g_clear_pointer (&priv->boxes_reverse, g_list_free);
+  if (priv->iio_proxy_watch_id != 0)
+    g_bus_unwatch_name (priv->iio_proxy_watch_id);
+  priv->iio_proxy_watch_id = 0;
 
   G_OBJECT_CLASS (cc_power_panel_parent_class)->dispose (object);
 }
@@ -947,6 +955,51 @@ sync_screen_brightness (CcPowerPanel *self)
     }
 }
 
+static void
+als_switch_changed (GtkSwitch    *sw,
+                    GParamSpec   *pspec,
+                    CcPowerPanel *panel)
+{
+  gboolean enabled;
+  enabled = gtk_switch_get_active (sw);
+  g_debug ("Setting ALS enabled %s", enabled ? "on" : "off");
+  g_settings_set_boolean (panel->priv->gsd_settings, "ambient-enabled", enabled);
+}
+
+static void
+als_enabled_state_changed (CcPowerPanel *self)
+{
+  CcPowerPanelPrivate *priv = self->priv;
+  gboolean enabled;
+  gboolean has_brightness = FALSE;
+  gboolean visible = FALSE;
+  GVariant *v;
+
+  v = g_dbus_proxy_get_cached_property (self->priv->screen_proxy, "Brightness");
+  if (v != NULL)
+    {
+      has_brightness = g_variant_get_int32 (v) >= 0.0;
+      g_variant_unref (v);
+    }
+
+  if (priv->iio_proxy != NULL)
+    {
+      v = g_dbus_proxy_get_cached_property (priv->iio_proxy, "HasAmbientLight");
+      if (v != NULL)
+        {
+          visible = g_variant_get_boolean (v);
+          g_variant_unref (v);
+        }
+    }
+
+  enabled = g_settings_get_boolean (priv->gsd_settings, "ambient-enabled");
+  g_debug ("ALS enabled: %s", enabled ? "on" : "off");
+  g_signal_handlers_block_by_func (priv->als_switch, als_switch_changed, self);
+  gtk_switch_set_active (GTK_SWITCH (priv->als_switch), enabled);
+  gtk_widget_set_visible (priv->als_row, visible && has_brightness);
+  g_signal_handlers_unblock_by_func (priv->als_switch, als_switch_changed, self);
+}
+
 static void
 on_screen_property_change (GDBusProxy *proxy,
                            GVariant   *changed_properties,
@@ -1077,7 +1130,7 @@ set_ac_battery_ui_mode (CcPowerPanel *self)
   UpDeviceKind kind;
 
   devices = up_client_get_devices (self->priv->up_client);
-  g_debug ("got %d devices from upower\n", devices ? devices->len : 0);
+  g_debug ("got %d devices from upower\n", devices->len);
 
   for (i = 0; devices != NULL && i < devices->len; i++)
     {
@@ -1098,8 +1151,6 @@ set_ac_battery_ui_mode (CcPowerPanel *self)
 
   self->priv->has_batteries = has_batteries;
 
-  gtk_widget_set_visible (self->priv->critical_battery_row, has_batteries);
-
   if (!has_batteries)
     {
       gtk_widget_hide (WID (self->priv->builder, "suspend_on_battery_switch"));
@@ -1434,6 +1485,44 @@ add_brightness_row (CcPowerPanel  *self,
   return row;
 }
 
+static void
+als_enabled_setting_changed (GSettings    *settings,
+                             const gchar  *key,
+                             CcPowerPanel *self)
+{
+  als_enabled_state_changed (self);
+}
+
+static void
+iio_proxy_appeared_cb (GDBusConnection *connection,
+                       const gchar *name,
+                       const gchar *name_owner,
+                       gpointer user_data)
+{
+  CcPowerPanel *self = CC_POWER_PANEL (user_data);
+  self->priv->iio_proxy =
+    g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+                                   G_DBUS_PROXY_FLAGS_NONE,
+                                   NULL,
+                                   "net.hadess.SensorProxy",
+                                   "/net/hadess/SensorProxy",
+                                   "net.hadess.SensorProxy",
+                                   NULL, NULL);
+  g_signal_connect_swapped (G_OBJECT (self->priv->iio_proxy), "g-properties-changed",
+                            G_CALLBACK (als_enabled_state_changed), self);
+  als_enabled_state_changed (self);
+}
+
+static void
+iio_proxy_vanished_cb (GDBusConnection *connection,
+                       const gchar *name,
+                       gpointer user_data)
+{
+  CcPowerPanel *self = CC_POWER_PANEL (user_data);
+  g_clear_object (&self->priv->iio_proxy);
+  als_enabled_state_changed (self);
+}
+
 static void
 add_power_saving_section (CcPowerPanel *self)
 {
@@ -1490,6 +1579,39 @@ add_power_saving_section (CcPowerPanel *self)
   gtk_container_add (GTK_CONTAINER (widget), row);
   gtk_size_group_add_widget (priv->row_sizegroup, row);
 
+  /* ambient light sensor */
+  priv->iio_proxy_watch_id =
+    g_bus_watch_name (G_BUS_TYPE_SYSTEM,
+                      "net.hadess.SensorProxy",
+                      G_BUS_NAME_WATCHER_FLAGS_NONE,
+                      iio_proxy_appeared_cb,
+                      iio_proxy_vanished_cb,
+                      self, NULL);
+  g_signal_connect (priv->gsd_settings, "changed",
+                    G_CALLBACK (als_enabled_setting_changed), self);
+  priv->als_row = row = no_prelight_row_new ();
+  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 50);
+  gtk_container_add (GTK_CONTAINER (row), box);
+  label = gtk_label_new (_("Automatic brightness"));
+  gtk_widget_set_halign (label, GTK_ALIGN_START);
+  gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
+  gtk_widget_set_margin_start (label, 20);
+  gtk_widget_set_margin_end (label, 20);
+  gtk_widget_set_margin_top (label, 6);
+  gtk_widget_set_margin_bottom (label, 6);
+  gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
+
+  priv->als_switch = sw = gtk_switch_new ();
+  gtk_widget_set_margin_start (sw, 20);
+  gtk_widget_set_margin_end (sw, 20);
+  gtk_widget_set_valign (sw, GTK_ALIGN_CENTER);
+  gtk_box_pack_start (GTK_BOX (box), sw, FALSE, TRUE, 0);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), sw);
+  gtk_container_add (GTK_CONTAINER (widget), row);
+  gtk_size_group_add_widget (priv->row_sizegroup, row);
+  g_signal_connect (G_OBJECT (priv->als_switch), "notify::active",
+                    G_CALLBACK (als_switch_changed), self);
+
   row = add_brightness_row (self, _("_Keyboard brightness"), &priv->kbd_brightness_scale);
   priv->kbd_brightness_row = row;
 
@@ -1803,9 +1925,6 @@ add_automatic_suspend_section (CcPowerPanel *self)
   gint value;
   GtkWidget *dialog;
   GtkWidget *combo;
-  GVariant *result;
-  GDBusConnection *connection;
-  const char *critical_battery_str;
 
   /* The default values for these settings are unfortunate for us;
    * timeout == 0, action == suspend means 'do nothing' - just
@@ -1868,7 +1987,6 @@ add_automatic_suspend_section (CcPowerPanel *self)
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 50);
   gtk_container_add (GTK_CONTAINER (row), box);
   label = gtk_label_new (_("_Automatic suspend"));
-  atk_object_set_name (ATK_OBJECT (gtk_widget_get_accessible (self->priv->automatic_suspend_row)), _("Automatic suspend"));
   gtk_widget_set_halign (label, GTK_ALIGN_START);
   gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
   gtk_widget_set_margin_start (label, 20);
@@ -1889,69 +2007,6 @@ add_automatic_suspend_section (CcPowerPanel *self)
   gtk_size_group_add_widget (priv->row_sizegroup, row);
   update_automatic_suspend_label (self);
 
-  priv->critical_battery_row = row = gtk_list_box_row_new ();
-  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-  gtk_container_add (GTK_CONTAINER (row), box);
-  label = gtk_label_new (_("When battery power is _critical"));
-  gtk_widget_set_halign (label, GTK_ALIGN_START);
-  gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
-  gtk_widget_set_margin_start (label, 20);
-  gtk_widget_set_margin_end (label, 20);
-  gtk_widget_set_margin_top (label, 6);
-  gtk_widget_set_margin_bottom (label, 6);
-  gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
-
-  connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, NULL);
-  result = g_dbus_connection_call_sync (connection,
-                                        "org.freedesktop.UPower",
-                                        "/org/freedesktop/UPower",
-                                        "org.freedesktop.UPower",
-                                        "GetCriticalAction",
-                                        NULL,
-                                        NULL,
-                                        G_DBUS_CALL_FLAGS_NONE,
-                                        -1,
-                                        NULL,
-                                        NULL);
-  g_object_unref (connection);
-
-  if (result)
-    {
-      GVariant *result_variant;
-      const char *str;
-
-      result_variant = g_variant_get_child_value (result, 0);
-      str = g_variant_get_string (result_variant, NULL);
-
-      if (g_strcmp0 (str, "Hibernate") == 0 ||
-          g_strcmp0 (str, "HybridSleep") == 0)
-        critical_battery_str = _("Hibernate");
-      else if (g_strcmp0 (str, "PowerOff") == 0)
-        critical_battery_str = _("Power Off");
-      else
-        {
-          g_debug ("Unexpected value returned by UPower for GetCriticalAction(): %s", str);
-          critical_battery_str = _("Power Off");
-        }
-
-      g_variant_unref(result_variant);
-      g_variant_unref(result);
-    }
-  else
-    {
-      g_debug ("GetCriticalAction() call failed, is UPower correctly installed?");
-      critical_battery_str = _("Power Off");
-    }
-
-  label = gtk_label_new (critical_battery_str);
-  gtk_widget_set_margin_start (label, 20);
-  gtk_widget_set_margin_end (label, 20);
-  gtk_widget_set_margin_top (label, 6);
-  gtk_widget_set_margin_bottom (label, 6);
-  gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
-
-  gtk_container_add (GTK_CONTAINER (widget), row);
-  gtk_size_group_add_widget (priv->row_sizegroup, row);
   gtk_widget_show_all (widget);
 
   dialog = priv->automatic_suspend_dialog;
